module Error(putLogLn,attempt, openLog, closeLog, tryElse, tryMost, tryMost_, catchMost, tryMap, tryMapM, first, handleMost) where

import Control.Exception
import IO
import Data.IORef
import IOExts
import Monad


ior :: IORef Handle
ior = unsafePerformIO $ newIORef stderr

openLog :: String -> IO ()
openLog fn = handle (\_ -> writeIORef ior stderr) $ do
	h <- openFile fn AppendMode
	hSetBuffering h NoBuffering
	writeIORef ior h

closeLog :: IO ()
closeLog = do
   h <- readIORef ior  
   when (h /= stderr) $ hClose h


putLogLn :: String -> IO ()
putLogLn s = readIORef ior >>= \h -> hPutStrLn h s

attempt :: IO a -> IO ()
attempt action = tryMost action >>= \x -> case x of 
    Left e -> putLogLn ("Exception Caught: " ++ show e)
    Right _ -> return ()


tryElse r x = tryMost x >>= \y -> case y of
    Left x -> (putLogLn ("tryElse ExceptionCaught:" ++ show x)) >> return r
    Right v -> return v

tryMost_ x = tryMost x >> return ()

tryMap :: (a -> b) -> [a] -> IO [b]
tryMap f xs = do 
    ys <- mapM (tryMost . evaluate . f ) xs
    return [y|(Right y) <- ys]
    
tryMapM :: (a -> IO b) -> [a] -> IO [b]
tryMapM f xs = do 
    ys <- mapM (tryMost . f ) xs
    return [y|(Right y) <- ys]

tryMost = Control.Exception.tryJust passKilled

passKilled (AsyncException ThreadKilled) = Nothing
passKilled x = Just x

catchMost = Control.Exception.catchJust passKilled
handleMost = Control.Exception.handleJust passKilled

first :: [IO a] -> IO a
first [] = fail "empty  argument to first"
first [x] = x
first (x:xs) = tryMost x >>= \z -> case z of
    Left e -> first xs
    Right v -> return v
